@@ -51,7 +51,7 @@ from playwright._impl._connection import (
 )
 from playwright._impl._console_message import ConsoleMessage
 from playwright._impl._download import Download
-from playwright._impl._element_handle import ElementHandle
+from playwright._impl._element_handle import ElementHandle, determine_screenshot_type
 from playwright._impl._errors import Error, TargetClosedError, is_target_closed_error
 from playwright._impl._event_context_manager import EventContextManagerImpl
 from playwright._impl._file_chooser import FileChooser
@@ -388,9 +388,7 @@ class Page(ChannelOwner):
         for frame in self._frames:
             if name and frame.name == name:
                 return frame
-            if url and url_matches(
-                self._browser_context._options.get("baseURL"), frame.url, url
-            ):
+            if url and url_matches(self._browser_context._base_url, frame.url, url):
                 return frame
 
         return None
@@ -447,12 +445,14 @@ class Page(ChannelOwner):
     async def is_hidden(
         self, selector: str, strict: bool = None, timeout: float = None
     ) -> bool:
-        return await self._main_frame.is_hidden(**locals_to_params(locals()))
+        # timeout is deprecated and does nothing
+        return await self._main_frame.is_hidden(selector=selector, strict=strict)
 
     async def is_visible(
         self, selector: str, strict: bool = None, timeout: float = None
     ) -> bool:
-        return await self._main_frame.is_visible(**locals_to_params(locals()))
+        # timeout is deprecated and does nothing
+        return await self._main_frame.is_visible(selector=selector, strict=strict)
 
     async def dispatch_event(
         self,
@@ -680,7 +680,7 @@ class Page(ChannelOwner):
         self._routes.insert(
             0,
             RouteHandler(
-                self._browser_context._options.get("baseURL"),
+                self._browser_context._base_url,
                 url,
                 handler,
                 True if self._dispatcher_fiber else False,
@@ -708,24 +708,21 @@ class Page(ChannelOwner):
         behavior: Literal["default", "ignoreErrors", "wait"] = None,
     ) -> None:
         self._routes = remaining
-        await self._update_interception_patterns()
-        if behavior is None or behavior == "default":
-            return
-        await asyncio.gather(
-            *map(
-                lambda route: route.stop(behavior),  # type: ignore
-                removed,
+        if behavior is not None and behavior != "default":
+            await asyncio.gather(
+                *map(
+                    lambda route: route.stop(behavior),  # type: ignore
+                    removed,
+                )
             )
-        )
+        await self._update_interception_patterns()
 
     async def route_web_socket(
         self, url: URLMatch, handler: WebSocketRouteHandlerCallback
     ) -> None:
         self._web_socket_routes.insert(
             0,
-            WebSocketRouteHandler(
-                self._browser_context._options.get("baseURL"), url, handler
-            ),
+            WebSocketRouteHandler(self._browser_context._base_url, url, handler),
         )
         await self._update_web_socket_interception_patterns()
 
@@ -803,6 +800,8 @@ class Page(ChannelOwner):
     ) -> bytes:
         params = locals_to_params(locals())
         if "path" in params:
+            if "type" not in params:
+                params["type"] = determine_screenshot_type(params["path"])
             del params["path"]
         if "mask" in params:
             params["mask"] = list(
@@ -1184,7 +1183,7 @@ class Page(ChannelOwner):
         # Note: we are creating Video object lazily, because we do not know
         # BrowserContextOptions when constructing the page - it is assigned
         # too late during launchPersistentContext.
-        if not self._browser_context._options.get("recordVideo"):
+        if not self._browser_context._videos_dir:
             return None
         return self._force_video()
 
@@ -1271,7 +1270,7 @@ class Page(ChannelOwner):
         def my_predicate(request: Request) -> bool:
             if not callable(urlOrPredicate):
                 return url_matches(
-                    self._browser_context._options.get("baseURL"),
+                    self._browser_context._base_url,
                     request.url,
                     urlOrPredicate,
                 )
@@ -1303,7 +1302,7 @@ class Page(ChannelOwner):
         def my_predicate(request: Response) -> bool:
             if not callable(urlOrPredicate):
                 return url_matches(
-                    self._browser_context._options.get("baseURL"),
+                    self._browser_context._base_url,
                     request.url,
                     urlOrPredicate,
                 )
