@@ -23,6 +23,7 @@ __export(channelOwner_exports, {
 module.exports = __toCommonJS(channelOwner_exports);
 var import_eventEmitter = require("./eventEmitter");
 var import_validator = require("../protocol/validator");
+var import_protocolMetainfo = require("../utils/isomorphic/protocolMetainfo");
 var import_clientStackTrace = require("./clientStackTrace");
 var import_stackTrace = require("../utils/isomorphic/stackTrace");
 class ChannelOwner extends import_eventEmitter.EventEmitter {
@@ -31,7 +32,6 @@ class ChannelOwner extends import_eventEmitter.EventEmitter {
     super(connection._platform);
     this._objects = /* @__PURE__ */ new Map();
     this._eventToSubscriptionMapping = /* @__PURE__ */ new Map();
-    this._isInternalType = false;
     this._wasCollected = false;
     this.setMaxListeners(0);
     this._connection = connection;
@@ -47,20 +47,14 @@ class ChannelOwner extends import_eventEmitter.EventEmitter {
     this._channel = this._createChannel(new import_eventEmitter.EventEmitter(connection._platform));
     this._initializer = initializer;
   }
-  markAsInternalType() {
-    this._isInternalType = true;
-  }
   _setEventToSubscriptionMapping(mapping) {
     this._eventToSubscriptionMapping = mapping;
   }
   _updateSubscription(event, enabled) {
     const protocolEvent = this._eventToSubscriptionMapping.get(String(event));
-    if (protocolEvent) {
-      this._wrapApiCall(async () => {
-        await this._channel.updateSubscription({ event: protocolEvent, enabled });
-      }, true).catch(() => {
+    if (protocolEvent)
+      this._channel.updateSubscription({ event: protocolEvent, enabled }).catch(() => {
       });
-    }
   }
   on(event, listener) {
     if (!this.listenerCount(event))
@@ -124,19 +118,19 @@ class ChannelOwner extends import_eventEmitter.EventEmitter {
       get: (obj, prop) => {
         if (typeof prop === "string") {
           const validator = (0, import_validator.maybeFindValidator)(this._type, prop, "Params");
+          const { internal } = import_protocolMetainfo.methodMetainfo.get(this._type + "." + prop) || {};
           if (validator) {
             return async (params) => {
               return await this._wrapApiCall(async (apiZone) => {
                 const validatedParams = validator(params, "", this._validatorToWireContext());
-                if (!apiZone.isInternal && !apiZone.reported) {
-                  apiZone.params = params;
+                if (!apiZone.internal && !apiZone.reported) {
                   apiZone.reported = true;
-                  this._instrumentation.onApiCallBegin(apiZone);
+                  this._instrumentation.onApiCallBegin(apiZone, { type: this._type, method: prop, params });
                   logApiCall(this._platform, this._logger, `=> ${apiZone.apiName} started`);
-                  return await this._connection.sendMessageToServer(this, prop, validatedParams, apiZone.apiName, apiZone.frames, apiZone.stepId);
+                  return await this._connection.sendMessageToServer(this, prop, validatedParams, apiZone);
                 }
-                return await this._connection.sendMessageToServer(this, prop, validatedParams, void 0, [], void 0);
-              });
+                return await this._connection.sendMessageToServer(this, prop, validatedParams, { internal: true });
+              }, { internal });
             };
           }
         }
@@ -146,18 +140,16 @@ class ChannelOwner extends import_eventEmitter.EventEmitter {
     channel._object = this;
     return channel;
   }
-  async _wrapApiCall(func, isInternal) {
+  async _wrapApiCall(func, options) {
     const logger = this._logger;
     const existingApiZone = this._platform.zones.current().data();
     if (existingApiZone)
       return await func(existingApiZone);
-    if (isInternal === void 0)
-      isInternal = this._isInternalType;
     const stackTrace = (0, import_clientStackTrace.captureLibraryStackTrace)(this._platform);
-    const apiZone = { apiName: stackTrace.apiName, frames: stackTrace.frames, isInternal, reported: false, userData: void 0, stepId: void 0 };
+    const apiZone = { title: options?.title, apiName: stackTrace.apiName, frames: stackTrace.frames, internal: options?.internal ?? false, reported: false, userData: void 0, stepId: void 0 };
     try {
       const result = await this._platform.zones.current().push(apiZone).run(async () => await func(apiZone));
-      if (!isInternal) {
+      if (!options?.internal) {
         logApiCall(this._platform, logger, `<= ${apiZone.apiName} succeeded`);
         this._instrumentation.onApiCallEnd(apiZone);
       }
@@ -171,17 +163,21 @@ class ChannelOwner extends import_eventEmitter.EventEmitter {
         e.stack = e.message + stackFrames;
       else
         e.stack = "";
-      if (!isInternal) {
+      if (!options?.internal) {
+        const recoveryHandlers = [];
         apiZone.error = e;
+        this._instrumentation.onApiCallRecovery(apiZone, e, recoveryHandlers);
+        for (const handler of recoveryHandlers) {
+          const recoverResult = await handler();
+          if (recoverResult.status === "recovered")
+            return recoverResult.value;
+        }
         logApiCall(this._platform, logger, `<= ${apiZone.apiName} failed`);
         this._instrumentation.onApiCallEnd(apiZone);
       }
       throw e;
     }
   }
-  _toImpl() {
-    return this._connection.toImpl?.(this);
-  }
   toJSON() {
     return {
       _type: this._type,
