@@ -25,7 +25,6 @@ __export(locator_exports, {
 });
 module.exports = __toCommonJS(locator_exports);
 var import_elementHandle = require("./elementHandle");
-var import_jsHandle = require("./jsHandle");
 var import_locatorGenerators = require("../utils/isomorphic/locatorGenerators");
 var import_locatorUtils = require("../utils/isomorphic/locatorUtils");
 var import_stringUtils = require("../utils/isomorphic/stringUtils");
@@ -101,6 +100,9 @@ class Locator {
   async evaluate(pageFunction, arg, options) {
     return await this._withElement((h) => h.evaluate(pageFunction, arg), { title: "Evaluate", timeout: options?.timeout });
   }
+  async _evaluateFunction(functionDeclaration, options) {
+    return await this._withElement((h) => h._evaluateFunction(functionDeclaration), { title: "Evaluate", timeout: options?.timeout });
+  }
   async evaluateAll(pageFunction, arg) {
     return await this._frame.$$eval(this._selector, pageFunction, arg);
   }
@@ -111,7 +113,7 @@ class Locator {
     return await this._frame.fill(this._selector, value, { strict: true, ...options });
   }
   async clear(options = {}) {
-    return await this.fill("", options);
+    await this._frame._wrapApiCall(() => this.fill("", options), { title: "Clear" });
   }
   async _highlight() {
     return await this._frame._highlight(this._selector);
@@ -190,11 +192,12 @@ class Locator {
   async blur(options) {
     await this._frame._channel.blur({ selector: this._selector, strict: true, ...options, timeout: this._frame._timeout(options) });
   }
-  async count() {
-    return await this._frame._queryCount(this._selector);
+  // options are only here for testing
+  async count(_options) {
+    return await this._frame._queryCount(this._selector, _options);
   }
-  async _generateLocatorString() {
-    return await this._withElement((h) => h._generateLocatorString(), { title: "Generate locator string", internal: true });
+  async _resolveSelector() {
+    return await this._frame._channel.resolveSelector({ selector: this._selector });
   }
   async getAttribute(name, options) {
     return await this._frame.getAttribute(this._selector, name, { strict: true, ...options });
@@ -286,12 +289,10 @@ class Locator {
     await this._frame._channel.waitForSelector({ selector: this._selector, strict: true, omitReturnValue: true, ...options, timeout: this._frame._timeout(options) });
   }
   async _expect(expression, options) {
-    const params = { selector: this._selector, expression, ...options, isNot: !!options.isNot };
-    params.expectedValue = (0, import_jsHandle.serializeArgument)(options.expectedValue);
-    const result = await this._frame._channel.expect(params);
-    if (result.received !== void 0)
-      result.received = (0, import_jsHandle.parseResult)(result.received);
-    return result;
+    return this._frame._expect(expression, {
+      ...options,
+      selector: this._selector
+    });
   }
   _inspect() {
     return this.toString();
