@@ -210,12 +210,9 @@ class WKBrowserContext extends import_browserContext.BrowserContext {
   possiblyUninitializedPages() {
     return this._wkPages().map((wkPage) => wkPage._page);
   }
-  async doCreateNewPage(markAsServerSideOnly) {
+  async doCreateNewPage() {
     const { pageProxyId } = await this._browser._browserSession.send("Playwright.createPage", { browserContextId: this._browserContextId });
-    const page = this._browser._wkPages.get(pageProxyId)._page;
-    if (markAsServerSideOnly)
-      page.markAsServerSideOnly();
-    return page;
+    return this._browser._wkPages.get(pageProxyId)._page;
   }
   async doGetCookies(urls) {
     const { cookies } = await this._browser._browserSession.send("Playwright.getAllCookies", { browserContextId: this._browserContextId });
